// CS 106 Winter 2018 Lab 06
// Your name
// Your ID

/////////////////////////////////////////////////////////////////////////
// This file contains the main program for a sketch that allows you to
// watch a sequence of moves that attempt to solve the Towers of Hanoi
// puzzle.  THIS IS NOT THE TAB YOU'RE LOOKING FOR.  You can basically
// ignore everything in this tab and go over to the "Moves" tab -- that's
// where you'll do all your work.
/////////////////////////////////////////////////////////////////////////





// A simple class that represents the act of moving a ring from one peg
// to another.  The pegs are numbers 0, 1 and 2, so a Move consists of
// the peg you're moving the ring from, and the peg you're moving it to.
// The fields should contain two different values from the set {0,1,2}.
class Move
{
  int from;
  int to;

  // Standard constructor.  Use this to build a new Move when you need 
  // one.  For example: 
  //   Move my_move = new Move( 1, 2 );
  Move( int fromIn, int toIn )
  {
    from = fromIn;
    to = toIn;
  }

  String toString()
  {
    return "" + from + " -> " + to;
  }
};

// The sequence of moves we'll be executing
Move[] seq;
// The number of rings we're working with
int game_rings;
// Where we currently are in the sequence
int cur;

// The contents of each of the pegs at this moment.
// An array of three arrays of integers, each containing
// the IDs of the rings on the peg, from bottom to top.
int[][] pegs;

void move( int from, int to )
{
  if ( PRINT_MOVES ) {
    println( "Move a disc from Peg " + from + " to Peg " + to );
  }
  seq = (Move[])append( seq, new Move( from, to ) );
}

void setup()
{
  size( 600, 220 );
  textSize( 100 );

  reset();
}

void reset()
{
  game_rings = NUM_RINGS;
  seq = new Move[0];

  // Ask the helper function to give us an array of moves.
  hanoiMoves( 0, 2, 1, NUM_RINGS );

  if ( seq.length == 0 ) {
    // No moves generated?  Assume the hanoiMoves() function isn't
    // implemented, and toss in the test sequence so that the 
    // sketch does something.
    game_rings = 3;
    generateTestMoves();
  }

  cur = 0;

  // Set the initial ring locations: all rings on Peg 0,
  // Pegs 1 and 2 empty.
  pegs = new int[3][];

  pegs[0] = new int[game_rings];
  for ( int idx = 0; idx < game_rings; ++idx ) {
    pegs[0][idx] = game_rings - idx;
  }
  pegs[1] = new int[0];
  pegs[2] = new int[0];
}

// Draw the rings associated with a single peg.
void drawRings( int[] arr )
{
  stroke( 0 );
  colorMode( HSB, 255 );

  float hh = float(height - 35) / game_rings;

  for ( int idx = 0; idx < arr.length; ++idx ) {
    int sz = arr[idx];
    float w = map( sz, 1, game_rings, 25, 95 );
    float hu = map( sz, 1, game_rings + 1, 0, 255 );
    fill( hu, 255, 255 );
    rect( -w, 200 - hh*(idx+1), 2 * w, hh, 10 );
  }

  colorMode( RGB, 255 );
}

void draw()
{
  background( 80 );
  fill( 255 );
  noStroke();
  rect( 90, 10, 20, 200 );
  rect( 290, 10, 20, 200 );
  rect( 490, 10, 20, 200 );
  rect( 0, 200, 600, 20 );

  for ( int idx = 0; idx < 3; ++idx ) {
    pushMatrix();
    translate( 100 + idx * 200, 0 );
    drawRings( pegs[idx] );
    popMatrix();
  }

  if ( hasWon() ) {
    fill( 0, 150 );
    noStroke();
    rect( 0, 0, width, height );

    fill( 255 );
    textAlign( CENTER );
    text( "YOU WON!", width/2, 140 );
  }
}

// Modify the peg arrays by applying the given move.  Catch
// illegal moves and print an error message to the console.
boolean applyMove( Move m )
{
  int[] pfrom = pegs[m.from];
  int[] pto = pegs[m.to];

  if ( pfrom.length == 0 ) {
    println( "You attempted to move a ring from an empty peg!" );
    return false;
  }

  int k = pfrom[pfrom.length-1];

  if ( (pto.length > 0) && (k > pto[pto.length-1]) ) {
    println( "You attempted to put a small ring on top of a large ring!" );
    return false;
  }

  pto = append( pto, k );
  pfrom = shorten( pfrom );

  pegs[m.from] = pfrom;
  pegs[m.to] = pto;

  return true;
}

boolean hasWon()
{
  return (cur == seq.length) && (pegs[2].length == game_rings);
}

// Press the space bar to advance through the sequence of moves.
void keyPressed()
{
  if ( key == ' ' ) {
    if ( cur < seq.length ) {
      Move m = seq[cur];
      applyMove( m );
      ++cur;
    }
  }
}